/**
   @file: im-engines.h
 */
/*
 * This file is part of libimengines
 *
 * Copyright (C) 2005 Nokia Corporation.
 *
 * Contact: Jakub Pavelek <jakub.pavelek@nokia.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 *
 */


#ifndef __IM_ENGINES_H__
#define __IM_ENGINES_H__

#include <glib.h>

/*
 * Common definitions
 */

/** Enum for supported languages and locales. It is basically a list of locales serving both. */
typedef enum t_IM_Locales {
  IM_af_ZA = 0,
  IM_am_ET,
  IM_ar_AE,
  IM_ar_BH,
  IM_ar_DZ,
  IM_ar_EG,
  IM_ar_IN,
  IM_ar_IQ,
  IM_ar_JO,
  IM_ar_KW,
  IM_ar_LB,
  IM_ar_LY,
  IM_ar_MA,
  IM_ar_OM,
  IM_ar_QA,
  IM_ar_SA,
  IM_ar_SD,
  IM_ar_SY,
  IM_ar_TN,
  IM_ar_YE,
  IM_az_AZ,
  IM_be_BY,
  IM_bg_BG,
  IM_bn_IN,
  IM_br_FR,
  IM_bs_BA,
  IM_ca_ES,
  IM_cs_CZ,
  IM_cy_GB,
  IM_da_DK,
  IM_de_AT,
  IM_de_BE,
  IM_de_CH,
  IM_de_DE,
  IM_de_LU,
  IM_el_GR,
  IM_en_AU,
  IM_en_BW,
  IM_en_CA,
  IM_en_DK,
  IM_en_GB,
  IM_en_HK,
  IM_en_IE,
  IM_en_IN,
  IM_en_NZ,
  IM_en_PH,
  IM_en_SG,
  IM_en_US,
  IM_en_ZA,
  IM_en_ZW,
  IM_eo_EO,
  IM_es_AR,
  IM_es_BO,
  IM_es_CL,
  IM_es_CO,
  IM_es_CR,
  IM_es_DO,
  IM_es_EC,
  IM_es_ES,
  IM_es_GT,
  IM_es_HN,
  IM_es_MX,
  IM_es_NI,
  IM_es_PA,
  IM_es_PE,
  IM_es_PR,
  IM_es_PY,
  IM_es_SV,
  IM_es_US,
  IM_es_UY,
  IM_es_VE,
  IM_et_EE,
  IM_eu_ES,
  IM_fa_IR,
  IM_fi_FI,
  IM_fo_FO,
  IM_fr_BE,
  IM_fr_CA,
  IM_fr_CH,
  IM_fr_FR,
  IM_fr_LU,
  IM_ga_IE,
  IM_gd_GB,
  IM_gl_ES,
  IM_gv_GB,
  IM_he_IL,
  IM_hi_IN,
  IM_hr_HR,
  IM_hu_HU,
  IM_hy_AM,
  IM_id_ID,
  IM_is_IS,
  IM_it_CH,
  IM_it_IT,
  IM_iw_IL,
  IM_ja_JP,
  IM_ka_GE,
  IM_kl_GL,
  IM_ko_KR,
  IM_kw_GB,
  IM_lt_LT,
  IM_lv_LV,
  IM_mi_NZ,
  IM_mk_MK,
  IM_mr_IN,
  IM_ms_MY,
  IM_mt_MT,
  IM_nl_BE,
  IM_nl_NL,
  IM_nn_NO,
  IM_no_NO,
  IM_oc_FR,
  IM_pl_PL,
  IM_pt_BR,
  IM_pt_PT,
  IM_ro_RO,
  IM_ru_RU,
  IM_ru_UA,
  IM_se_NO,
  IM_sk_SK,
  IM_sl_SI,
  IM_sq_AL,
  IM_sr_YU,
  IM_sv_FI,
  IM_sv_SE,
  IM_ta_IN,
  IM_te_IN,
  IM_tg_TJ,
  IM_th_TH,
  IM_ti_ER,
  IM_ti_ET,
  IM_tl_PH,
  IM_tr_TR,
  IM_tt_RU,
  IM_uk_UA,
  IM_ur_PK,
  IM_uz_UZ,
  IM_vi_VN,
  IM_wa_BE,
  IM_yi_US,
  IM_zh_CN,
  IM_zh_HK,
  IM_zh_SG,
  IM_zh_TW
} IM_Locales;

/** 
Enum for the return status of IM operation. It is a union of all possible status codes of
the used IM engines. Some error codes are shared though 
*/
typedef enum t_IM_Status {
  IM_STATUS_UNKNOWN = -1,
  IM_STATUS_OK = 0,
  IM_STATUS_FAILED,
  IM_STATUS_OUT_OF_RANGE,
  IM_STATUS_NOT_ENOUGHT_MEMORY,
  IM_STATUS_OUT_OF_LIMITS,
  IM_STATUS_OUT_OF_DISKSPACE,
  IM_STATUS_NOT_SUPPORTED,
  IM_STATUS_HWR_NO_DICTIONARY,
  IM_STATUS_HWR_NO_RESULTS,
  IM_STATUS_HWR_INCOMPLETE_RESULTS,
  IM_STATUS_HWR_FINAL_RESULTS,
  IM_STATUS_HWR_TRAINING_CONFLICT,
  IM_STATUS_HWR_TRAINING_CHARACTER_NOT_ALLOWED,
  IM_STATUS_HWR_TRAINING_TOO_MANY_STROKES,
  IM_STATUS_HWR_TRAINING_NO_STROKES,
  IM_STATUS_HWR_LIBRARY_CORRUPT,
  IM_STATUS_HWR_IO_FAILURE  
    
} IM_Status;

/*
 * Hand-Writting (HWR) specific part
 */
/** 
Enum for the Handwritting conversion. Latin will possibly not need any of these while
Chinese might require most of them 
*/
typedef enum t_HWR_Conversion {
  HWR_CONVERT_UNDEFINED = -1,
  HWR_CONVERT_NONE = 0,
  HWR_CONVERT_TO_UPPER,
  HWR_CONVERT_TO_LOWER,
  HWR_CONVERT_GB_TO_BIG5,
  HWR_CONVERT_BIG5_TO_GB,
} HWR_Conversion;

typedef struct t_HWR_Point {
  short x;
  short y;
} HWR_Point;

typedef struct {
  HWR_Point *points;
  int number_points;
} HWR_Points;

typedef struct t_HWR_Results {
  int *codes;
  int number_results;
} HWR_Results;

typedef struct {
  int start_x, start_y, end_x, end_y;
} HWR_Box;

/**
Enumerated values for character type. This is used for the Handwritting to tell the 
engine about what kind of character we are trying to draw.
*/
typedef enum t_HWR_CharacterClass {
  HWR_CHARACTER_CLASS_UNDEFINED = -1,
  HWR_CHARACTER_CLASS_ALPHA_UPPERCASE = 0,
  HWR_CHARACTER_CLASS_ALPHA_LOWERCASE,
  HWR_CHARACTER_CLASS_NUMERIC,
  HWR_CHARACTER_CLASS_SPECIAL,
  HWR_CHARACTER_CLASS_MIXED,
  HWR_CHARACTER_CLASS_SHORTCUT_GESTURE,
} HWR_CharacterClass;


/** 
Enumerated types for the script. Some languages can be written in different scripts, 
e.g. Kurdish or Russian (Cyrillic vs Latin). 
*/
typedef enum t_HWR_ScriptClass {
  HWR_SCRIPT_CLASS_UNDEFINED = -1,
  HWR_SCRIPT_CLASS_ANY = 0,
  HWR_SCRIPT_CLASS_ARABIC,
  HWR_SCRIPT_CLASS_ARMENIAN,
  HWR_SCRIPT_CLASS_BENGALI,
  HWR_SCRIPT_CLASS_BOPOMOFO,
  HWR_SCRIPT_CLASS_BRAILLE,
  HWR_SCRIPT_CLASS_BUHID,
  HWR_SCRIPT_CLASS_CANADIANABORIGINAL,
  HWR_SCRIPT_CLASS_CHEROKEE,
  HWR_SCRIPT_CLASS_COMMON,
  HWR_SCRIPT_CLASS_CYPRIOT,
  HWR_SCRIPT_CLASS_CYRILLIC,
  HWR_SCRIPT_CLASS_DESERET,
  HWR_SCRIPT_CLASS_DEVANAGARI,
  HWR_SCRIPT_CLASS_ETHIOPIC,
  HWR_SCRIPT_CLASS_GEORGIAN,
  HWR_SCRIPT_CLASS_GOTHIC,
  HWR_SCRIPT_CLASS_GREEK,
  HWR_SCRIPT_CLASS_GUJARATI,
  HWR_SCRIPT_CLASS_GURMUKHI,
  HWR_SCRIPT_CLASS_HAN,
  HWR_SCRIPT_CLASS_HANGUL,
  HWR_SCRIPT_CLASS_HANUNOO,
  HWR_SCRIPT_CLASS_HEBREW,
  HWR_SCRIPT_CLASS_HIRAGANA,
  HWR_SCRIPT_CLASS_INHERITED,
  HWR_SCRIPT_CLASS_KANNADA,
  HWR_SCRIPT_CLASS_KATAKANA,
  HWR_SCRIPT_CLASS_KHMER,
  HWR_SCRIPT_CLASS_LAO,
  HWR_SCRIPT_CLASS_LATIN,
  HWR_SCRIPT_CLASS_LIMBU,
  HWR_SCRIPT_CLASS_LINEARB,
  HWR_SCRIPT_CLASS_MALAYALAM,
  HWR_SCRIPT_CLASS_MONGOLIAN,
  HWR_SCRIPT_CLASS_MYANMAR,
  HWR_SCRIPT_CLASS_OGHAM,
  HWR_SCRIPT_CLASS_OLDITALIC,
  HWR_SCRIPT_CLASS_ORIYA,
  HWR_SCRIPT_CLASS_OSMANYA,
  HWR_SCRIPT_CLASS_RUNIC,
  HWR_SCRIPT_CLASS_SHAVIAN,
  HWR_SCRIPT_CLASS_SINHALA,
  HWR_SCRIPT_CLASS_SYRIAC,
  HWR_SCRIPT_CLASS_TAGALOG,
  HWR_SCRIPT_CLASS_TAGBANWA,
  HWR_SCRIPT_CLASS_TAILE,
  HWR_SCRIPT_CLASS_TAMIL,
  HWR_SCRIPT_CLASS_TELUGU,
  HWR_SCRIPT_CLASS_THAANA,
  HWR_SCRIPT_CLASS_THAI,
  HWR_SCRIPT_CLASS_TIBETAN,
  HWR_SCRIPT_CLASS_UGARITIC,
  HWR_SCRIPT_CLASS_YI
} HWR_ScriptClass;

typedef struct {
  char character[6];
  gboolean is_trained_character;
} HWR_Conflicts;

typedef struct {
  gchar character[6];
} HWR_Character;

typedef struct {
  HWR_Character *list;
  int number_of_chars;
} HWR_CharacterList;

/* 
 * Word-Prediction /WP) specific part
 */
/** The maximal number of candidates we want to see from the engine */
#define WP_MAX_CANDIDATES 10 
/** The maximal lenght of each candidate */
#define WP_MAX_CANDIDATE_LENGTH 128 
/** Initial/added size of dictionaries */
#define DEFAULT_DICT_SIZE 1024
/** The maximal dictionary size (disk space liniter) of 10kB */
#define IM_WP_PLUGIN_MAX_DICT_SIZE 10240


typedef struct t_WP_Candidates {
  int number_of_candidates;
  char candidate[WP_MAX_CANDIDATES][WP_MAX_CANDIDATE_LENGTH];
} WP_Candidates;

/*
 * Function prototypes for HWR
 */

/** A specific code marking the first shortcut */
#define IM_FIRST_SHORTCUT_CODE 0x1ED2
/** A specific code marking the last shortcut */
#define IM_LAST_SHORTCUT_CODE 0x1EE1

IM_Status
HWR_Init(void);

IM_Status
HWR_Destroy (void);

IM_Status
HWR_SetLanguage(const IM_Locales language, 
                const HWR_CharacterClass character_class,
                HWR_ScriptClass *SCRIPT_CLASS);

IM_Status
HWR_GetLanguage(IM_Locales *language);

IM_Status
HWR_SetCharacterClass (HWR_CharacterClass class);

IM_Status
HWR_GetCharacterClass (HWR_CharacterClass *class);

IM_Status
HWR_SetMaxCandidateNum(const int number);

IM_Status
HRW_GetMaxCandidateNum(int *number);

IM_Status
HWR_SetAndRecognize(HWR_Points *points, HWR_Results *results);

IM_Status
HWR_Recognize(HWR_Results *results);

IM_Status
HWR_SetInputboxSize(HWR_Box);

IM_Status
HWR_GetEngineVersion(char **version);

IM_Status
HWR_GetDictVersion(char **version);

IM_Status
HWR_SetCharOutFlag(const HWR_Conversion conversion);

IM_Status
HWR_LoadUserLibrary (const char *path);

IM_Status 
HWR_StartTraining (void);

IM_Status 
HWR_FinishTraining (const char*, HWR_Conflicts *);

IM_Status 
HWR_CancelTraining (void);

IM_Status 
HWR_GetModels (const char *character, int *number_of_models, HWR_Points **, HWR_Box);

IM_Status 
HWR_GetLibraryModels (const char *character, int *number_of_models, HWR_Points **, HWR_Box);

IM_Status 
HWR_GetTrainedCharacters (HWR_CharacterList *list);

IM_Status 
HWR_DeleteModel (const char *character, int index);

/*
 * Function prototypes for WP
 */

IM_Status
WP_Init(void);

IM_Status
WP_SetPredictionLanguage(IM_Locales language);

IM_Status
WP_AttachUsedWordsDictionary(const gboolean use_dictionary);

IM_Status
WP_AttachPersonalDictionary(const gboolean use_dictionary);

IM_Status
WP_AttachDictionary(IM_Locales language);

IM_Status
WP_DetachUsedWordsDictionary(void);

IM_Status
WP_DetachPersonalDictionary(void);

IM_Status
WP_DetachDictionary(void);

IM_Status
WP_GetCandidates(const char *word, WP_Candidates *candidates);

IM_Status
WP_SetMaxCandidates(const int number);

IM_Status
WP_GetMaxCandidates(int *number);

gboolean
WP_WordExistInUsedWordsDictionary(const char *word);

gboolean
WP_WordExistInPersonalDictionary(const char *word);

gboolean
WP_WordExistInDictionary(const char *word);

IM_Status
WP_HitWordUsedWordsDictionary(const char *word);

IM_Status
WP_HitWordPersonalDictionary(const char *word);

IM_Status
WP_HitWordDictionary(const char *word);

IM_Status
WP_DeleteWordUsedWordsDictionary(const char *word);

IM_Status
WP_DeleteWordPersonalDictionary(const char *word);

IM_Status
WP_DeleteWordDictionary(const char *word);

IM_Status
WP_AddWordUsedWordsDictionary(const char *word);
                                                                                         
IM_Status
WP_AddWordPersonalDictionary(const char *word);
                                                                                         
IM_Status
WP_AddWordDictionary(const char *word);

IM_Status
WP_SaveUsedWordsDictionary(void);

IM_Status
WP_SavePersonalDictionary(void);

IM_Status
WP_SaveDictionary(void);


#endif
