/**
   @file: im-hwr-plugin.h
 */
/*
 * This file is part of libimengines
 *
 * Copyright (C) 2005 Nokia Corporation.
 *
 * Contact: Jakub Pavelek <jakub.pavelek@nokia.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 *
 */


#ifndef __IM_MODULE_H__
#define __IM_MODULE_H__

#include "im-engines.h"
typedef struct {
  IM_Status (*HWR_Plugin_Destroy) (void);
  IM_Status (*HWR_Plugin_SetHwrLanguage) (const IM_Locales language, 
                               const HWR_CharacterClass, 
                               HWR_ScriptClass *);
  IM_Status (*HWR_Plugin_GetHwrLanguage) (IM_Locales *language);
  IM_Status (*HWR_Plugin_SetCharacterClass) (HWR_CharacterClass class);
  IM_Status (*HWR_Plugin_GetCharacterClass) (HWR_CharacterClass *class);
  IM_Status (*HWR_Plugin_SetMaxCandidateNum) (const int number);
  IM_Status (*HWR_Plugin_GetMaxCandidateNum) (int *number);
  IM_Status (*HWR_Plugin_SetAndRecognize) (HWR_Points *points, HWR_Results *results);
  IM_Status (*HWR_Plugin_Recognize) (HWR_Results *results);
  IM_Status (*HWR_Plugin_SetInputboxSize) (HWR_Box);
  IM_Status (*HWR_Plugin_GetEngineVersion) (char **version);
  IM_Status (*HWR_Plugin_GetDictVersion) (char **version);
  IM_Status (*HWR_Plugin_SetCharOutFlag) (const HWR_Conversion conversion);
  IM_Status (*HWR_Plugin_StartTraining) (void);
  IM_Status (*HWR_Plugin_FinishTraining) (const char*, HWR_Conflicts *);
  IM_Status (*HWR_Plugin_CancelTraining) (void);
  IM_Status (*HWR_Plugin_GetModels) (const char *character, int *number_of_models, HWR_Points **strokes, HWR_Box);
  IM_Status (*HWR_Plugin_GetLibraryModels) (const char *character, int *number_of_models, HWR_Points **strokes, HWR_Box);
  IM_Status (*HWR_Plugin_GetTrainedCharacters) (HWR_CharacterList *list);
  IM_Status (*HWR_Plugin_DeleteModel) (const char *character, int index);
  IM_Status (*HWR_Plugin_LoadUserLibrary) (const char *path);
} IM_HWR_Plugin;

IM_HWR_Plugin*
im_hwr_plugin_init(void);


IM_Status
HWR_Plugin_Destroy (void);

IM_Status
HWR_Plugin_SetHwrLanguage(const IM_Locales language,
                           const HWR_CharacterClass character_class,
                           HWR_ScriptClass *script_class);

IM_Status
HWR_Plugin_GetHwrLanguage(IM_Locales *language);

IM_Status
HWR_Plugin_SetCharacterClass (HWR_CharacterClass class);

IM_Status
HWR_Plugin_GetCharacterClass (HWR_CharacterClass *class);

IM_Status
HWR_Plugin_SetMaxCandidateNum(const int number);

IM_Status
HWR_Plugin_GetMaxCandidateNum(int *number);

IM_Status
HWR_Plugin_SetAndRecognize(HWR_Points *points, HWR_Results *results);

IM_Status
HWR_Plugin_Recognize(HWR_Results *results);

IM_Status
HWR_Plugin_SetInputboxSize(HWR_Box);

IM_Status
HWR_Plugin_GetEngineVersion(char **version);

IM_Status
HWR_Plugin_GetDictVersion(char **version);

IM_Status
HWR_Plugin_SetCharOutFlag(const HWR_Conversion conversion);

IM_Status
HWR_Plugin_LoadUserLibrary (const char *);

IM_Status 
HWR_Plugin_StartTraining (void);

IM_Status 
HWR_Plugin_FinishTraining (const char*, HWR_Conflicts *);

IM_Status 
HWR_Plugin_CancelTraining (void);

IM_Status 
HWR_Plugin_GetModels (const char *character, int *number_of_models, HWR_Points **, HWR_Box);

IM_Status 
HWR_Plugin_GetLibraryModels (const char *character, int *number_of_models, HWR_Points **, HWR_Box);

IM_Status 
HWR_Plugin_GetTrainedCharacters (HWR_CharacterList *list);

IM_Status 
HWR_Plugin_DeleteModel (const char *character, int index);
#endif
