/**
   @file: im-wp-plugin.h
 */

/*
 * This file is part of libimengines
 *
 * Copyright (C) 2005 Nokia Corporation.
 *
 * Contact: Jakub Pavelek <jakub.pavelek@nokia.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 *
 */

#ifndef __IM_WP_PLUGIN_H__
#define __IM_WP_PLUGIN_H__

#include "im-engines.h"

typedef struct t_IM_WP_Plugin {
  IM_Status (*WP_Plugin_SetPredictionLanguage) (IM_Locales language);
  IM_Status (*WP_Plugin_AttachUsedWordsDictionary) (const gboolean use_dictionary);
  IM_Status (*WP_Plugin_AttachPersonalDictionary) (const gboolean use_dictionary);
  IM_Status (*WP_Plugin_AttachDictionary) (IM_Locales language);
  IM_Status (*WP_Plugin_DetachUsedWordsDictionary) (void);
  IM_Status (*WP_Plugin_DetachPersonalDictionary) (void);
  IM_Status (*WP_Plugin_DetachDictionary) (void);
  IM_Status (*WP_Plugin_GetCandidates) (const char *word, WP_Candidates *candidates);
  IM_Status (*WP_Plugin_SetMaxCandidates) (const int number);
  IM_Status (*WP_Plugin_GetMaxCandidates) (int *number);
  gboolean  (*WP_Plugin_WordExistInUsedWordsDictionary) (const char *word);
  gboolean  (*WP_Plugin_WordExistInPersonalDictionary) (const char *word);
  gboolean  (*WP_Plugin_WordExistInDictionary) (const char *word);
  IM_Status (*WP_Plugin_HitWordUsedWordsDictionary) (const char *word);
  IM_Status (*WP_Plugin_HitWordPersonalDictionary) (const char *word);
  IM_Status (*WP_Plugin_HitWordDictionary) (const char *word);
  IM_Status (*WP_Plugin_DeleteWordUsedWordsDictionary) (const char *word);
  IM_Status (*WP_Plugin_DeleteWordPersonalDictionary) (const char *word);
  IM_Status (*WP_Plugin_DeleteWordDictionary) (const char *word);
  IM_Status (*WP_Plugin_AddWordUsedWordsDictionary) (const char *word);
  IM_Status (*WP_Plugin_AddWordPersonalDictionary) (const char *word);
  IM_Status (*WP_Plugin_AddWordDictionary) (const char *word);
  IM_Status (*WP_Plugin_SaveUsedWordsDictionary) (void);
  IM_Status (*WP_Plugin_SavePersonalDictionary) (void);
  IM_Status (*WP_Plugin_SaveDictionary) (void);

} IM_WP_Plugin;

IM_WP_Plugin*
im_wp_plugin_init(void);

IM_Status
WP_Plugin_SetPredictionLanguage(IM_Locales language);

IM_Status
WP_Plugin_AttachUsedWordsDictionary(const gboolean use_dictionary);

IM_Status
WP_Plugin_AttachPersonalDictionary(const gboolean use_dictionary);

IM_Status
WP_Plugin_AttachDictionary(IM_Locales language);

IM_Status
WP_Plugin_DetachUsedWordsDictionary(void);

IM_Status
WP_Plugin_DetachPersonalDictionary(void);

IM_Status
WP_Plugin_DetachDictionary(void);

IM_Status
WP_Plugin_GetCandidates(const char *word, WP_Candidates *candidates);

IM_Status
WP_Plugin_SetMaxCandidates(const int number);

IM_Status
WP_Plugin_GetMaxCandidates(int *number);

gboolean
WP_Plugin_WordExistInUsedWordsDictionary(const char *word);

gboolean
WP_Plugin_WordExistInPersonalDictionary(const char *word);

gboolean
WP_Plugin_WordExistInDictionary(const char *word);

IM_Status
WP_Plugin_HitWordUsedWordsDictionary(const char *word);

IM_Status
WP_Plugin_HitWordPersonalDictionary(const char *word);

IM_Status
WP_Plugin_HitWordDictionary(const char *word);

IM_Status
WP_Plugin_DeleteWordUsedWordsDictionary(const char *word);

IM_Status
WP_Plugin_DeleteWordPersonalDictionary(const char *word);

IM_Status
WP_Plugin_DeleteWordDictionary(const char *word);

IM_Status
WP_Plugin_AddWordUsedWordsDictionary(const char *word);
                                                                                               
IM_Status
WP_Plugin_AddWordPersonalDictionary(const char *word);
                                                                                               
IM_Status
WP_Plugin_AddWordDictionary(const char *word);

IM_Status
WP_Plugin_SaveUsedWordsDictionary(void);
                                                                                                
IM_Status
WP_Plugin_SavePersonalDictionary(void);
                                                                                                
IM_Status
WP_Plugin_SaveDictionary(void);


#endif
